@@ -14,7 +14,7 @@ use super::Uname;
 use std::borrow::Cow;
 use std::ffi::CStr;
 use std::io;
-use std::mem;
+use std::mem::MaybeUninit;
 
 macro_rules! cstr2cow {
     ($v:expr) => {
@@ -32,9 +32,11 @@ impl PlatformInfo {
     /// Creates a new instance of `PlatformInfo`.  This function *should* never fail.
     pub fn new() -> io::Result<Self> {
         unsafe {
-            #[allow(deprecated)]
-            let mut uts: utsname = mem::uninitialized();
-            if uname(&mut uts) != -1 {
+            let mut uts = MaybeUninit::<utsname>::uninit();
+            if uname(uts.as_mut_ptr()) != -1 {
+                // SAFETY: `uts` was initialized
+                let uts = uts.assume_init();
+
                 Ok(Self { inner: uts })
             } else {
                 Err(io::Error::last_os_error())
@@ -63,4 +65,15 @@ impl Uname for PlatformInfo {
     fn machine(&self) -> Cow<str> {
         cstr2cow!(self.inner.machine)
     }
+
+    fn osname(&self) -> Cow<str> {
+        Cow::Borrowed(crate::HOST_OS_NAME)
+    }
+}
+
+#[test]
+fn test_osname() {
+    let info = PlatformInfo::new().unwrap();
+    println!("osname = '{}'", info.osname());
+    assert_eq!(PlatformInfo::new().unwrap().osname(), crate::HOST_OS_NAME);
 }
