@@ -4,7 +4,15 @@
 //
 // For the full copyright and license information, please view the LICENSE file
 // that was distributed with this source code.
-//
+
+// Note: there no standardization of values for platform info (or `uname`), so mimic some current practices
+// busybox-v1.35.0 * `busybox uname -a` => "Windows_NT HOSTNAME 10.0 19044 x86_64 MS/Windows"
+// python-v3.8.3 => `uname_result(system='Windows', node='HOSTNAME', release='10', version='10.0.19044', machine='AMD64')`
+
+// refs:
+// [NT Version Info](https://en.wikipedia.org/wiki/Windows_NT) @@ <https://archive.is/GnnvF>
+// [NT Version Info (summary)](https://simple.wikipedia.org/wiki/Windows_NT) @@ <https://archive.is/T2StZ>
+// [NT Version Info (detailed)](https://en.wikipedia.org/wiki/Comparison_of_Microsoft_Windows_versions#Windows_NT) @@ <https://archive.is/FSkhj>
 
 extern crate winapi;
 
@@ -21,7 +29,7 @@ use std::borrow::Cow;
 use std::ffi::{CStr, OsStr, OsString};
 use std::io;
 use std::iter;
-use std::mem;
+use std::mem::{self, MaybeUninit};
 use std::os::windows::ffi::{OsStrExt, OsStringExt};
 use std::path::PathBuf;
 use std::ptr;
@@ -45,12 +53,20 @@ struct VS_FIXEDFILEINFO {
     dwFileDateLS: DWORD,
 }
 
+#[derive(Debug)]
+struct WinOSVersionInfo {
+    os_name: String,
+    release: String,
+    version: String,
+}
+
 /// `PlatformInfo` handles retrieving information for the current platform (Windows in this case).
 pub struct PlatformInfo {
     sysinfo: SYSTEM_INFO,
     nodename: String,
     release: String,
     version: String,
+    osname: String,
 }
 
 impl PlatformInfo {
@@ -58,19 +74,21 @@ impl PlatformInfo {
     /// it is possible for this function to fail.
     pub fn new() -> io::Result<Self> {
         unsafe {
-            #[allow(deprecated)]
-            let mut sysinfo = mem::uninitialized();
+            let mut sysinfo = MaybeUninit::<SYSTEM_INFO>::uninit();
+            GetNativeSystemInfo(sysinfo.as_mut_ptr());
+            // SAFETY: `sysinfo` was initialized
+            let sysinfo = sysinfo.assume_init();
 
-            GetNativeSystemInfo(&mut sysinfo);
+            let version_info = Self::version_info()?;
 
-            let (version, release) = Self::version_info()?;
             let nodename = Self::computer_name()?;
 
             Ok(Self {
                 sysinfo,
                 nodename,
-                version,
-                release,
+                version: version_info.version,
+                release: version_info.release,
+                osname: format!("{} ({})", crate::HOST_OS_NAME, version_info.os_name),
             })
         }
     }
@@ -97,7 +115,10 @@ impl PlatformInfo {
 
     // NOTE: the only reason any of this has to be done is Microsoft deprecated GetVersionEx() and
     //       it is now basically useless for us on Windows 8.1 and Windows 10
-    unsafe fn version_info() -> io::Result<(String, String)> {
+    // ref: <https://learn.microsoft.com/en-us/windows/win32/api/sysinfoapi/nf-sysinfoapi-getversionexw> @@ <https://archive.is/bYAwT>
+    // ref: <https://learn.microsoft.com/en-us/windows/win32/api/winnt/ns-winnt-osversioninfoexw> @@ <https://archive.is/n4hBb>
+    unsafe fn version_info() -> io::Result<WinOSVersionInfo> {
+        // busybox-v1.35.0 * `busybox uname -a` => "Windows_NT HOSTNAME 10.0 19044 x86_64 MS/Windows"
         let dll_wide: Vec<WCHAR> = OsStr::new("ntdll.dll")
             .encode_wide()
             .chain(iter::once(0))
@@ -114,42 +135,44 @@ impl PlatformInfo {
                 osinfo.dwOSVersionInfoSize = mem::size_of::<RTL_OSVERSIONINFOEXW>() as _;
 
                 if func(&mut osinfo) == STATUS_SUCCESS {
-                    let version = String::from_utf16_lossy(
-                        osinfo.szCSDVersion.split(|&v| v == 0).next().unwrap(),
-                    );
-                    let release = Self::determine_release(
-                        osinfo.dwMajorVersion,
-                        osinfo.dwMinorVersion,
-                        osinfo.wProductType,
-                        osinfo.wSuiteMask,
-                    );
-
-                    return Ok((version, release));
+                    return Ok(WinOSVersionInfo {
+                        os_name: Self::determine_os_name(
+                            osinfo.dwMajorVersion,
+                            osinfo.dwMinorVersion,
+                            osinfo.dwBuildNumber,
+                            osinfo.wProductType,
+                            osinfo.wSuiteMask.into(),
+                        ),
+                        release: format!("{}.{}", osinfo.dwMajorVersion, osinfo.dwMinorVersion),
+                        version: format!("{}", osinfo.dwBuildNumber),
+                    });
                 }
             }
         }
 
         // as a last resort, try to get the relevant info by loading the version info from a system
         // file (specifically Kernel32.dll)
+        // Note: this file version may be just the current "base" version and not the actual most up-to-date version info
+        // * eg: kernel32.dll (or ntdll.dll) version => "10.0.19041.2130" _vs_ `cmd /c ver` => "10.0.19044.2364"
         Self::version_info_from_file()
     }
 
-    fn version_info_from_file() -> io::Result<(String, String)> {
+    fn version_info_from_file() -> io::Result<WinOSVersionInfo> {
         use self::winapi::um::sysinfoapi;
 
         let pathbuf = Self::get_kernel32_path()?;
 
         let file_info = Self::get_file_version_info(pathbuf)?;
-        let (major, minor) = Self::query_version_info(file_info)?;
+        let (major, minor, build, _revision) = Self::query_version_info(file_info)?;
 
-        #[allow(deprecated)]
-        let mut info: OSVERSIONINFOEXW = unsafe { mem::uninitialized() };
+        // SAFETY: this is valid
+        let mut info = unsafe { mem::zeroed::<OSVERSIONINFOEXW>() };
         info.wSuiteMask = VER_SUITE_WH_SERVER as WORD;
         info.wProductType = VER_NT_WORKSTATION;
 
         let mask = unsafe { sysinfoapi::VerSetConditionMask(0, VER_SUITENAME, VER_EQUAL) };
         let suite_mask = if unsafe { VerifyVersionInfoW(&mut info, VER_SUITENAME, mask) } != 0 {
-            VER_SUITE_WH_SERVER as USHORT
+            VER_SUITE_WH_SERVER
         } else {
             0
         };
@@ -162,10 +185,11 @@ impl PlatformInfo {
             0
         };
 
-        Ok((
-            String::new(),
-            Self::determine_release(major, minor, product_type, suite_mask),
-        ))
+        Ok(WinOSVersionInfo {
+            os_name: Self::determine_os_name(major, minor, build, product_type, suite_mask),
+            release: format!("{}.{}", major, minor),
+            version: format!("{}", build),
+        })
     }
 
     fn get_kernel32_path() -> io::Result<PathBuf> {
@@ -221,10 +245,9 @@ impl PlatformInfo {
         }
     }
 
-    fn query_version_info(buffer: Vec<u8>) -> io::Result<(ULONG, ULONG)> {
+    fn query_version_info(buffer: Vec<u8>) -> io::Result<(DWORD, DWORD, DWORD, DWORD)> {
         let mut block_size = 0;
-        #[allow(deprecated)]
-        let mut block = unsafe { mem::uninitialized() };
+        let mut block = ptr::null_mut();
 
         let sub_block: Vec<_> = OsStr::new("\\")
             .encode_wide()
@@ -242,28 +265,39 @@ impl PlatformInfo {
             return Err(io::Error::last_os_error());
         }
 
+        // SAFETY: `block` was replaced with a non-null pointer
         let info = unsafe { &*(block as *const VS_FIXEDFILEINFO) };
 
         Ok((
             HIWORD(info.dwProductVersionMS) as _,
             LOWORD(info.dwProductVersionMS) as _,
+            HIWORD(info.dwProductVersionLS) as _,
+            LOWORD(info.dwProductVersionLS) as _,
         ))
     }
 
-    fn determine_release(
-        major: ULONG,
-        minor: ULONG,
-        product_type: UCHAR,
-        suite_mask: USHORT,
+    fn determine_os_name(
+        major: DWORD,
+        minor: DWORD,
+        build: DWORD,
+        product_type: BYTE,
+        suite_mask: DWORD,
     ) -> String {
-        let mut name = match major {
+        // [NT Version Info (detailed)](https://en.wikipedia.org/wiki/Comparison_of_Microsoft_Windows_versions#Windows_NT) @@ <https://archive.is/FSkhj>
+        let default_name = if product_type == VER_NT_WORKSTATION {
+            format!("{} {}.{}", "Windows", major, minor)
+        } else {
+            format!("{} {}.{}", "Windows Server", major, minor)
+        };
+
+        let name = match major {
             5 => match minor {
                 0 => "Windows 2000",
                 1 => "Windows XP",
                 2 if product_type == VER_NT_WORKSTATION => "Windows XP Professional x64 Edition",
-                2 if suite_mask as UINT == VER_SUITE_WH_SERVER => "Windows Home Server",
+                2 if suite_mask == VER_SUITE_WH_SERVER => "Windows Home Server",
                 2 => "Windows Server 2003",
-                _ => "",
+                _ => &default_name,
             },
             6 => match minor {
                 0 if product_type == VER_NT_WORKSTATION => "Windows Vista",
@@ -274,25 +308,25 @@ impl PlatformInfo {
                 2 => "Windows 8",
                 3 if product_type != VER_NT_WORKSTATION => "Windows Server 2012 R2",
                 3 => "Windows 8.1",
-                _ => "",
+                _ => &default_name,
             },
             10 => match minor {
-                0 if product_type != VER_NT_WORKSTATION => "Windows Server 2016",
-                _ => "",
+                0 if product_type == VER_NT_WORKSTATION && (build >= 22000) => "Windows 11",
+                0 if product_type != VER_NT_WORKSTATION && (14000..17000).contains(&build) => {
+                    "Windows Server 2016"
+                }
+                0 if product_type != VER_NT_WORKSTATION && (17000..19000).contains(&build) => {
+                    "Windows Server 2019"
+                }
+                0 if product_type != VER_NT_WORKSTATION && (build >= 20000) => {
+                    "Windows Server 2022"
+                }
+                _ => "Windows 10",
             },
-            _ => "",
+            _ => &default_name,
         };
 
-        // we're doing this down here so we don't have to copy this into multiple branches
-        if name.is_empty() {
-            name = if product_type == VER_NT_WORKSTATION {
-                "Windows"
-            } else {
-                "Windows Server"
-            };
-        }
-
-        format!("{} {}.{}", name, major, minor)
+        name.to_string()
     }
 }
 
@@ -302,7 +336,7 @@ impl Uname for PlatformInfo {
 
         // XXX: if Rust ever works on Windows CE and winapi has the VER_PLATFORM_WIN32_CE
         //      constant, we should probably check for that
-        Cow::from("WindowsNT")
+        Cow::from("Windows_NT") // prior art from `busybox` and MS (from std::env::var("OS"))
     }
 
     fn nodename(&self) -> Cow<str> {
@@ -347,6 +381,10 @@ impl Uname for PlatformInfo {
 
         Cow::from(arch_str)
     }
+
+    fn osname(&self) -> Cow<str> {
+        Cow::from(self.osname.as_str())
+    }
 }
 
 #[cfg(test)]
@@ -380,12 +418,16 @@ fn is_wow64() -> bool {
 
 #[test]
 fn test_sysname() {
-    assert_eq!(PlatformInfo::new().unwrap().sysname(), "WindowsNT");
+    let info = PlatformInfo::new().unwrap();
+    let expected: String = std::env::var("OS").unwrap_or_else(|_| String::from("Windows_NT"));
+    println!("sysname = '{}'", info.sysname());
+    assert_eq!(info.sysname(), expected);
 }
 
 #[test]
 fn test_machine() {
-    let target = if cfg!(target_arch = "x86_64") || (cfg!(target_arch = "x86") && is_wow64()) {
+    let is_wow64 = is_wow64();
+    let target = if cfg!(target_arch = "x86_64") || (cfg!(target_arch = "x86") && is_wow64) {
         vec!["x86_64"]
     } else if cfg!(target_arch = "x86") {
         vec!["i386", "i486", "i586", "i686"]
@@ -406,6 +448,146 @@ fn test_machine() {
 
     let info = PlatformInfo::new().unwrap();
 
-    println!("{}", info.machine());
+    println!("machine = '{}'", info.machine());
     assert!(target.contains(&&*info.machine()));
 }
+
+#[test]
+fn test_osname() {
+    let info = PlatformInfo::new().unwrap();
+    println!("osname = '{}'", info.osname());
+    assert!(info.osname().starts_with(crate::HOST_OS_NAME));
+}
+
+#[test]
+fn test_version_vs_version() {
+    let version_via_dll = unsafe { PlatformInfo::version_info().unwrap() };
+    let version_via_file = PlatformInfo::version_info_from_file().unwrap();
+
+    println!("version (via dll) = '{:#?}'", version_via_dll);
+    println!("version (via file) = '{:#?}'", version_via_file);
+
+    assert_eq!(version_via_dll.os_name, version_via_file.os_name);
+    assert_eq!(version_via_dll.release, version_via_file.release);
+    // the "version" portions may differ, but should have only slight variation
+    // * assume that "version" is convertible to u32 + "version" from file is always earlier/smaller and may differ only below the thousands digit
+    // * ref: [NT Version Info (detailed)](https://en.wikipedia.org/wiki/Comparison_of_Microsoft_Windows_versions#Windows_NT) @@ <https://archive.is/FSkhj>
+    assert!(
+        (version_via_dll.version.parse::<u32>().unwrap()
+            - version_via_file.version.parse::<u32>().unwrap())
+            < 1000
+    );
+}
+
+#[test]
+fn test_known_os_names() {
+    // ref: [NT Version Info (detailed)](https://en.wikipedia.org/wiki/Comparison_of_Microsoft_Windows_versions#Windows_NT) @@ <https://archive.is/FSkhj>
+    assert_eq!(
+        PlatformInfo::determine_os_name(3, 1, 528, VER_NT_WORKSTATION, VER_SUITE_PERSONAL),
+        "Windows 3.1"
+    );
+    assert_eq!(
+        PlatformInfo::determine_os_name(3, 5, 807, VER_NT_WORKSTATION, VER_SUITE_PERSONAL),
+        "Windows 3.5"
+    );
+    assert_eq!(
+        PlatformInfo::determine_os_name(3, 51, 1057, VER_NT_WORKSTATION, VER_SUITE_PERSONAL),
+        "Windows 3.51"
+    );
+    assert_eq!(
+        PlatformInfo::determine_os_name(4, 0, 1381, VER_NT_WORKSTATION, VER_SUITE_PERSONAL),
+        "Windows 4.0"
+    );
+    assert_eq!(
+        PlatformInfo::determine_os_name(5, 0, 2195, VER_NT_WORKSTATION, VER_SUITE_PERSONAL),
+        "Windows 2000"
+    );
+    assert_eq!(
+        PlatformInfo::determine_os_name(5, 1, 2600, VER_NT_WORKSTATION, VER_SUITE_PERSONAL),
+        "Windows XP"
+    );
+    assert_eq!(
+        PlatformInfo::determine_os_name(5, 2, 3790, VER_NT_WORKSTATION, VER_SUITE_PERSONAL),
+        "Windows XP Professional x64 Edition"
+    );
+    assert_eq!(
+        PlatformInfo::determine_os_name(5, 2, 3790, VER_NT_SERVER, VER_SUITE_WH_SERVER),
+        "Windows Home Server"
+    );
+    assert_eq!(
+        PlatformInfo::determine_os_name(5, 2, 3790, VER_NT_SERVER, VER_SUITE_SMALLBUSINESS),
+        "Windows Server 2003"
+    );
+    assert_eq!(
+        PlatformInfo::determine_os_name(5, 2, 3790, VER_NT_SERVER, VER_SUITE_SMALLBUSINESS),
+        "Windows Server 2003"
+    );
+    assert_eq!(
+        PlatformInfo::determine_os_name(6, 0, 6000, VER_NT_WORKSTATION, VER_SUITE_PERSONAL),
+        "Windows Vista"
+    );
+    assert_eq!(
+        PlatformInfo::determine_os_name(6, 0, 6001, VER_NT_SERVER, VER_SUITE_SMALLBUSINESS),
+        "Windows Server 2008"
+    );
+    assert_eq!(
+        PlatformInfo::determine_os_name(6, 1, 7600, VER_NT_WORKSTATION, VER_SUITE_PERSONAL),
+        "Windows 7"
+    );
+    assert_eq!(
+        PlatformInfo::determine_os_name(6, 1, 7600, VER_NT_SERVER, VER_SUITE_SMALLBUSINESS),
+        "Windows Server 2008 R2"
+    );
+    assert_eq!(
+        PlatformInfo::determine_os_name(6, 2, 9200, VER_NT_SERVER, VER_SUITE_SMALLBUSINESS),
+        "Windows Server 2012"
+    );
+    assert_eq!(
+        PlatformInfo::determine_os_name(6, 2, 9200, VER_NT_WORKSTATION, VER_SUITE_PERSONAL),
+        "Windows 8"
+    );
+    assert_eq!(
+        PlatformInfo::determine_os_name(6, 3, 9600, VER_NT_WORKSTATION, VER_SUITE_PERSONAL),
+        "Windows 8.1"
+    );
+    assert_eq!(
+        PlatformInfo::determine_os_name(6, 3, 9600, VER_NT_SERVER, VER_SUITE_SMALLBUSINESS),
+        "Windows Server 2012 R2"
+    );
+    assert_eq!(
+        PlatformInfo::determine_os_name(10, 0, 10240, VER_NT_WORKSTATION, VER_SUITE_PERSONAL),
+        "Windows 10"
+    );
+    assert_eq!(
+        PlatformInfo::determine_os_name(10, 0, 17134, VER_NT_WORKSTATION, VER_SUITE_PERSONAL),
+        "Windows 10"
+    );
+    assert_eq!(
+        PlatformInfo::determine_os_name(10, 0, 19141, VER_NT_WORKSTATION, VER_SUITE_PERSONAL),
+        "Windows 10"
+    );
+    assert_eq!(
+        PlatformInfo::determine_os_name(10, 0, 19145, VER_NT_WORKSTATION, VER_SUITE_PERSONAL),
+        "Windows 10"
+    );
+    assert_eq!(
+        PlatformInfo::determine_os_name(10, 0, 14393, VER_NT_SERVER, VER_SUITE_SMALLBUSINESS),
+        "Windows Server 2016"
+    );
+    assert_eq!(
+        PlatformInfo::determine_os_name(10, 0, 17763, VER_NT_SERVER, VER_SUITE_SMALLBUSINESS),
+        "Windows Server 2019"
+    );
+    assert_eq!(
+        PlatformInfo::determine_os_name(10, 0, 20348, VER_NT_SERVER, VER_SUITE_SMALLBUSINESS),
+        "Windows Server 2022"
+    );
+    assert_eq!(
+        PlatformInfo::determine_os_name(10, 0, 22000, VER_NT_WORKSTATION, VER_SUITE_PERSONAL),
+        "Windows 11"
+    );
+    assert_eq!(
+        PlatformInfo::determine_os_name(10, 0, 22621, VER_NT_WORKSTATION, VER_SUITE_PERSONAL),
+        "Windows 11"
+    );
+}
